"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.compose = exports.composeMany = void 0;
const composition_1 = require("../entities/composition");
/**
 * Specify on the array properties.
 *
 * Typescript compiles to Javascript before execution. Since Javascript doesn't feature types,
 * the type information of an array is lost at runtime, hence making it impossible to parse an
 * array of objects to proper types.
 * @param type Type parameter that denotes the class to which an array of object needs to be casted to.
 */
function composeMany(type) {
    return (target, propertyKey) => {
        checkAndInitializeCompositions(target);
        target.compositions.push(new composition_1.Composition(propertyKey, type, true));
    };
}
exports.composeMany = composeMany;
/**
 * Specify on the object properties.
 *
 * Typescript compiles to Javascript before execution. Since Javascript doesn't feature types,
 * the type information is lost at runtime, hence making it impossible to parse an
 * objects to proper types.
 * @param type Type parameter that denotes the class to which an object needs to be casted to.
 */
function compose(type) {
    return (target, propertyKey) => {
        checkAndInitializeCompositions(target);
        target.compositions.push(new composition_1.Composition(propertyKey, type, false));
    };
}
exports.compose = compose;
function checkAndInitializeCompositions(target) {
    if (!target.hasOwnProperty('compositions')) {
        Object.defineProperty(target, 'compositions', {
            value: new Array()
        });
    }
}
